@@ -26,7 +26,7 @@ use Data::Dumper;
 require Exporter;
 @ISA = qw(Exporter);
 @EXPORT = qw(create_ikc_channel);
-$VERSION = '0.03';
+$VERSION = '0.09';
 
 sub DEBUG { 0 }
 
@@ -470,7 +470,8 @@ sub channel_send
     $request->{rsvp}->{kernel}||=$heap->{kernel_name}
             if $request->{rsvp};
 
-    return $heap->{'wheel_client'}->put($request);
+    $heap->{'wheel_client'}->put($request);
+    return 1;
 }
 
 #----------------------------------------------------
