@@ -25,7 +25,7 @@ use Carp;
 require Exporter;
 @ISA = qw(Exporter);
 @EXPORT = qw(create_ikc_client);
-$VERSION = '0.03';
+$VERSION = '0.09';
 
 sub DEBUG { 0 }
 
@@ -57,7 +57,7 @@ sub create_ikc_client
     };
 
     if($@) {
-        $error=$@;
+        $self{error}=$error=$@;
         return;
     }
     my $self=bless \%self, $package;
@@ -131,7 +131,7 @@ sub connect
     };
     if($@)
     {
-        $error=$@;
+        $self->{error}=$error=$@;
         return;
     } 
     $remote->{socket}=$sock;
@@ -158,7 +158,7 @@ sub ping
         my $pong=$self->_response($rsvp);
         return 1 if $pong and $pong eq 'PONG';
     }; 
-    $error=$@ if $@;
+    $self->{error}=$error=$@ if $@;
     $self->{remote}{connected}=$ret;
     return $ret;
 }
@@ -191,7 +191,7 @@ sub post
     unless(ref $spec or $spec=~m(^poe:)) {
         
         unless($self->{remote}{name}) {
-            $error="Attempting to post $spec to unknown kernel";
+            $self->{error}=$error="Attempting to post $spec to unknown kernel";
             carp $error;
             return;
         }
@@ -204,7 +204,7 @@ sub post
         1;
     };
     if($@) {
-        $error=$@;
+        $self->{error}=$error=$@;
         return;
     }
     return $ret;
@@ -228,7 +228,7 @@ sub call
         return $self->_response($rsvp);
     };
     if($@) {
-        $error=$@;
+        $self->{error}=$error=$@;
         return;
     }
     return $ret;
@@ -254,7 +254,7 @@ sub post_respond
         return $self->_response($rsvp);
     };
     if($@) {
-        $error=$@;
+        $self->{error}=$error=$@;
         return;
     }
     return $ret;
