@@ -25,7 +25,7 @@ use POE::Component::IKC::Specifier;
 require Exporter;
 @ISA = qw(Exporter);
 @EXPORT = qw(create_ikc_responder $ikc);
-$VERSION = '0.03';
+$VERSION = '0.09';
 
 sub DEBUG { 0 }
 
@@ -199,6 +199,7 @@ sub do_you_have
        ))
     {
         $ses->{kernel}||=$kernel->ID;       # make sure we uniquely identify 
+        DEBUG && print "Allowed (we are $ses->{kernel})\n";
         return $ses;                        # this session
     } else
     {
@@ -238,8 +239,7 @@ use strict;
 use Carp;
 use POE::Component::IKC::Specifier;
 use POE::Component::IKC::Proxy;
-
-*ARG0=\&POE::Component::IKC::Responder::ARG0;
+use POE qw(Session);
 
 use Data::Dumper;
 
@@ -522,6 +522,7 @@ sub send_msg
     }
 
     DEBUG2 && print specifier_name($to), " sent to $count kernel(s).\n";
+    DEBUG and do {warn "send_msg failed!" unless $count};
     return $count;
 }
 
@@ -694,7 +695,7 @@ sub subscribe
                                     # Create the subscription receipt state
         $kernel->state($unique.$spec, 
                        sub {
-                          _subscribe_receipt($self, $unique, $spec, $_[&ARG0])
+                          _subscribe_receipt($self, $unique, $spec, $_[ARG0])
                        } 
                       );
         $kernel->delay($unique.$spec, 60);  # timeout
@@ -889,10 +890,9 @@ sub _start
     if($caller)                             # foreign session wants return
     {
         my $ret=$kernel->call(@$args);
-        if(defined $ret)
-        {
+        if(defined $ret) {
+            DEBUG && print "Posted response '$ret' to ", Dumper $caller;
             $kernel->post('IKC', 'post', $caller, $ret) ;
-            # DEBUG && print "Posted response to ", Dumper $caller;
         }
     } else
     {
