@@ -21,16 +21,16 @@ use POE qw(Wheel::ListenAccept Wheel::SocketFactory);
 use POE::Component::IKC::Channel;
 use POE::Component::IKC::Responder;
 
-use POSIX qw(ECHILD EAGAIN);
+use POSIX qw(ECHILD EAGAIN WNOHANG);
 
 require Exporter;
 
 @ISA = qw(Exporter);
 @EXPORT = qw(create_ikc_server);
-$VERSION = '0.08';
+$VERSION = '0.09';
 
 sub DEBUG { 0 }
-
+sub DEBUG_USR2 { 1 }
 
 ###############################################################################
 #----------------------------------------------------
@@ -43,15 +43,17 @@ sub create_ikc_server
     $params{ip}||='0.0.0.0';            # INET_ANY
     $params{port}||=603;                # POE! (almost :)
     create_ikc_responder();
-    new POE::Session( 
+    POE::Session->new( 
                     _start   => \&server_start,
                     _stop    => \&server_stop,
                     error    => \&server_error,
                     'accept' => \&server_accept,
+                    'other_signal' => \&other_signal,
                     'signal' => \&server_signal,
                     'fork'   => \&server_fork,
                     'retry'  => \&server_retry,
                     'waste_time' => \&server_waste,
+                    'babysit' => \&server_babysit,
                     [\%params],
                   );
 }
@@ -67,7 +69,7 @@ sub _select_define
         $state=$heap->{wheel}->{'state_accept'};
         
         unless(defined $state) {
-            die "No 'state_accept' in $heap->{wheel}, possible redefinition of POE internals.\n";
+            die "No 'state_accept' in $heap->{wheel}, possible redefinition of POE internals or failure to bind to port.\n";
         }
     }
     my $c=0;
@@ -88,7 +90,7 @@ sub server_start
 {
     my($heap, $params, $kernel) = @_[HEAP, ARG0, KERNEL];
 
-    DEBUG && print "Server starting $params->{ip}:$params->{port}.\n";
+    DEBUG && print "$$: Server starting $params->{ip}:$params->{port}.\n";
                                         # create a socket factory
     $heap->{wheel} = new POE::Wheel::SocketFactory
     ( BindPort       => $params->{port},
@@ -108,9 +110,11 @@ sub server_start
 
     $kernel->sig('CHLD', 'signal');
     $kernel->sig('INT', 'signal');
+    DEBUG_USR2 and $kernel->sig('USR2', 'other_signal');
                                         # keep track of children
     $heap->{children} = {};
     $heap->{'failed forks'} = 0;
+    $heap->{verbose}=$params->{verbose}||0;
     $heap->{"max connections"}=$params->{connections}||1;
                                         # change behavior for children
     $heap->{'is a child'} = 0;
@@ -118,31 +122,80 @@ sub server_start
         $kernel->yield('fork');
     }
     $kernel->yield('waste_time');    
+    $kernel->yield('babysit')           if $params->{babysit};    
     return;
 }
 
 #------------------------------------------------------------------------------
+# This event keeps this POE kernel alive
 sub server_waste
 {
     my($kernel, $heap)=@_[KERNEL, HEAP];
     return if $heap->{'is a child'};
 
     if($heap->{'die'}) {
-        DEBUG and print "$$: Orderly shutdown\n";
+        DEBUG and warn "$$: Orderly shutdown\n";
     } else {
-        $kernel->delay('waste_time', 1) 
+        $kernel->delay('waste_time', 60);
     }
     return;
+}
     
+#------------------------------------------------------------------------------
+# Babysit the child processes
+sub server_babysit
+{
+    my($kernel, $heap)=@_[KERNEL, HEAP];
+
+    return if $heap->{'die'} or             # don't scan if we are dieing
+              $heap->{'is a child'};        # or if we are a child
+
     my @children=keys %{$heap->{children}};
-    print "$$: Scanning children ", 
-            join(", ", @children), "\n";
-    foreach (@children) {
-        next if kill 0, $_;
-        warn "$$: child $_ has gone missing: $!\n";
+    DEBUG and warn  "$$: Scanning children ", join(", ", sort @children), "\n";
+    my(%missing, $state, $stateT);
+    foreach my $pid (@children) {
+        unless(open STATUS, "/proc/$pid/status") {
+            warn "$$: Unable to open /proc/$pid/status!  Is the child missing: $!\n";
+            $missing{$pid}=1;
+        } else {
+            undef($state);
+            undef($stateT);
+            while(<STATUS>) {
+                next unless /^State:\s*(\w)\s*\((\w+)\)/;
+                ($state, $stateT)=($1,$2);
+                last;
+            }
+            close STATUS or warn $!;
+
+            if(defined $state) {
+                if($state eq 'Z') {
+                    my $t=waitpid($pid, POSIX::WNOHANG());
+                    if($t==$pid) {
+                        # process was reaped, now fake a SIGCHLD
+                        $kernel->yield('signal', 'CHLD', $pid, $?);
+                        DEBUG and warn "$$: Faking a CHLD for $pid\n";            
+                    } else {
+                        $heap->{verbose} and warn "$$: $pid is a $state ($stateT) and couldn't be reaped.\n";
+                        $missing{$pid}=1;
+                    }
+                } elsif($state eq 'R' or $state eq 'S') {
+                    # do nothing
+                } else {
+                    $heap->{verbose} and warn "$$: $pid has unknown state $state ($stateT)\n";
+                }
+            } else {
+                $heap->{verbose} and warn "$$: Can't find status for $pid!\n";
+                $missing{$pid}=1;
+            }
+        }
     }
 
-    $kernel->delay('waste_time', 1);
+    foreach my $pid (keys %missing) {
+        $kernel->yield('signal', 'CHLD', $pid, 0);
+        DEBUG and warn "$$: Faking a CHLD for $pid MIA\n";            
+    }
+
+    $kernel->delay('babysit', 60);
     return;
 }
 
@@ -254,11 +307,12 @@ sub server_fork
         $heap->{children}->{$pid} = 1;
         DEBUG &&
             print( "$$: master server forked a new child.  children: (",
-                    join(' ', keys %{$heap->{children}}), ")\n"
+                    join(' ', sort keys %{$heap->{children}}), ")\n"
                  );
     }
                                         # child becomes a child server
     else {
+        $heap->{verbose} and warn "$$: Created ", scalar localtime, "\n";
         $heap->{'is a child'}   = 1;        # don't allow fork
         $heap->{children}       = { };      # don't kill child processes
                                             # limit sessions, then die off
@@ -320,8 +374,9 @@ sub server_signal
         if (delete $heap->{children}->{$pid}) {
             DEBUG &&
                     print( "$$: master caught SIGCHLD for $pid.  children: (",
-                                join(' ', keys %{$heap->{children}}), ")\n"
+                                join(' ', sort keys %{$heap->{children}}), ")\n"
                         );
+            $heap->{verbose} and warn "$$: Child $pid exited.\n";
             $kernel->yield('fork') unless $heap->{'die'};
         } else {
             warn "$$: CHLD for a child of someone else.\n";
@@ -343,7 +398,13 @@ sub server_signal
 }
 
 
-
+sub other_signal
+{
+#    return unless DEBUG;
+    my ($signal, $pid) = @_[ARG0, ARG1];
+    $pid||='';
+    warn "$$: signal $signal $pid\n";
+}
 
 
 1;
