@@ -9,7 +9,7 @@
 BEGIN { $| = 1; print "1..1\n"; }
 END {print "not ok 1\n" unless $loaded;}
 use POE::Component::IKC::Server;
-use POE::Component::IKC::Daemon;
+use POE::Component::IKC::Channel;
 use POE::Component::IKC::Client;
 $loaded = 1;
 print "ok 1\n";
